#########################################################################
#########################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# Linear Normal model to analyse abundance indices of Thiof fish in Senegal
# Chapter  6

######################################################################################
######################################################################################


rm(list = ls())

library(rbugs)
library(boot)
library("BRugs")
library(coda)

# load a function needed to save mcmc results
source("function.save.mcmc.R")

# Create repertory "results" and "results_forecast" in the working directory
dir.create(path = "results", recursive = TRUE, showWarnings = TRUE)


# ------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------
# Explore data and make graphics (no bayesian model here !)
# ------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------

# Data not used in the BUGS program (only for graphics)

data <- read.table(file = "data_lm_AbundanceIndicesThiof_table.txt", header = T, sep = "")
k <- source("data_lm_AbundanceIndicesThiof.txt")$value$k
n.years <- source("data_lm_AbundanceIndicesThiof.txt")$value$n.years
years <- source("data_lm_AbundanceIndicesThiof.txt")$value$years.pred





# Compute catch per unit effort and join the new variable to the table
# ----------------------------------------------------

data$CPUE <- data$Catches/data$Effort



# Declare variables as factors (except Year)
# ----------------------------------------------------

data$Month <- as.factor(data$Month)
data$Season <- as.factor(data$Season)
data$Zone <- as.factor(data$Zone)
data$Arbor <- as.factor(data$Arbor)



# System of constraints "effects sun to zero"
# ----------------------------------------------------

contrasts(data$Month) <- contr.sum(levels(data$Month))
contrasts(data$Season) <- contr.sum(levels(data$Season))
contrasts(data$Zone) <- contr.sum(levels(data$Zone))
contrasts(data$Arbor) <- contr.sum(levels(data$Arbor))



# Tabulate factors
# ----------------------------------------------------

levels(data$Month)
levels(data$Season)
levels(data$Zone)
levels(data$Arbor)



size.text = 1.3


# Plot residuals of a linear model without log-transformation of CPUEs
# ----------------------------------------------------


lm1 <- lm(CPUE ~ Zone + Season + Year, data=data)

# windows()

res <- 6
name_figure <- "ResidualsHeteroscedasticity.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mfrow = c(1,1), bty="n")

plot(fitted(lm1),residuals(lm1),pch=19, xlab="", ylab = "", cex=0.5, xaxt="n", yaxt="n")

axis(side = 1, tick = T, cex.axis=1, las=1) 
axis(side = 2, tick = T, cex.axis=1, las=1) 

mtext(side=1, text = "Fitted CPUE", line=3, cex=size.text, font = 1)
mtext(side=2, text = "Residuals", line=3, cex=size.text, font = 1)


dev.off()




# Some graph illustrating the variation of CPUE for each level of factors
# ----------------------------------------------------

# windows()

res <- 6
name_figure <- "ExplorationLogCPUE.png"
png(filename = name_figure, height = 500*res, width = 750*res, res=72*res)

par(mfrow = c(2,2), bty="n")
ymin = -5
ymax = 5

boxplot(	log(data$CPUE) ~ as.factor(data$Year), 
		ylim=c(ymin,ymax), ylab = "", xlab = "", xaxt="n", yaxt="n", bty="n",
		col = "lightgrey", outline = F, at=1:26, boxwex =0.6)

axis(side = 1, tick = T, labels = 1974:1999, cex.axis=1, las=1, at=1:26) 
axis(side = 2, tick = T, labels = T, cex.axis=1, las=1) 

mtext(side=1, text = "Year", line=3, cex=size.text, font = 1)
mtext(side=2, text = "log(CPUE)", line=3, cex=size.text, font = 1)



interaction.plot(	x.factor = as.factor(data$Year), trace.factor = data$Season, response = log(data$CPUE), fun = mean,  
  	   		ylim=c(ymin+3,ymax-1), ylab = "", xlab = "", xaxt="n", yaxt="n", main = "", bty="n",
			legend = F)

legend(	legend = c("Cold season","Warm season"),
 		col = c("black","black"), 	
 		lty = c(2,1), lwd = c(1,1), x = "topright", cex = 1, bty ="n") 

axis(side = 1, tick = T, labels = 1974:1999, cex.axis=1, las=1, at=1:26) 
axis(side = 2, tick = T, labels = T, cex.axis=1, las=1) 

mtext(side=1, text = "Year", line=3, cex=size.text, font = 1)
mtext(side=2, text = "log(CPUE)", line=3, cex=size.text, font = 1)


boxplot(	log(data$CPUE) ~ as.factor(data$Zone), ylim=c(ymin,ymax), 
		ylab = "", xlab = "", xaxt="n", yaxt="n", main = "", bty="n", 
		col = "lightgrey", outline = F, at=1:4)

axis(side = 1, tick = T, labels = c(1,2,3,4), cex.axis=1, las=1, at=1:4) 
axis(side = 2, tick = T, labels = T, cex.axis=1, las=1) 

mtext(side=1, text = "Zone", line=3, cex=size.text, font = 1)
mtext(side=2, text = "log(CPUE)", line=3, cex=size.text, font = 1)

dev.off()




# ------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------
# Run BUGS model
# ------------------------------------------------------------------------------------
# ------------------------------------------------------------------------------------


# ------------------------------------------------------------------------------------
# Run the routine used to 
# - generate the matrix of covariates X used for linear modeling Y = X * theta + eps
# - generate the Precision matrix P0 used for the Zellner prior on the parameters Theta
# ------------------------------------------------------------------------------------

source("Generate_DataFile_BUGSFormated_lmThiof.R")

# ------------------------------------------------------------------------------------
# MCMC config.
# (be careful, this model runs very very slowly ...)
# ------------------------------------------------------------------------------------

# Number iteration for the "burn in" phase
n.burnIn <- 3000
# Number iteration for inferences
n.iter <- 5000
thin <- 1
# Number of chains
n.chains <- 3


# ------------------------------------------------------------------------------------
# Run model M1 
# ------------------------------------------------------------------------------------

# Load/check model, load data and inits

model <- "model_lm_AbundanceIndicesThiof.txt"
data <- "data_lm_AbundanceIndicesThiof.txt"

modelCheck(model)  
modelData(data)   
modelCompile(numChains=n.chains)      
modelGenInits()

# Burn-in period

modelUpdate(n.burnIn) 

# Inferences

monitor <- c("theta","pred.seas1","pred.seas2","Epred.seas1","Epred.seas2")
samplesSet(monitor) 
modelUpdate(numUpdates = n.iter, thin = thin)


# -------------------------------------------------------------------------
# Have a quick look on results
# -------------------------------------------------------------------------

# Have a look on MCMC chains of parameters theta

windows()
Results <- samplesHistory("theta", mfrow = c(3, 1))

# Get posterior statistics of parameters theta

samplesStats("theta") 


# ------------------------------------------------------------
# Store MCMC samples of theta in a data.frame
# ------------------------------------------------------------

setwd("./results")
for(i in 1:length(monitor)) {function.save.mcmc(var=monitor[i],write=TRUE)}
setwd("../")

# ------------------------------------------------------------
# ------------------------------------------------------------
#                         GRAPHICS
# ------------------------------------------------------------
# ------------------------------------------------------------

# Load saved MCMC samples

setwd("./results")

theta <- as.matrix(read.table(file = "theta", header = T, sep = ""))
pred.seas1 <- as.matrix(read.table(file = "pred.seas1", header = T, sep = ""))
pred.seas2 <- as.matrix(read.table(file = "pred.seas2", header = T, sep = ""))
Epred.seas1 <- as.matrix(read.table(file = "Epred.seas1", header = T, sep = ""))
Epred.seas2 <- as.matrix(read.table(file = "Epred.seas2", header = T, sep = ""))

setwd("../")

# ------------------------------------------------------------
# Boxplots of log-predicted CPUE
# ------------------------------------------------------------

# windows()

res <- 6
name_figure <- "PostPredictiveLogCPUE.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(bty="n", mar = c(5,4,1,1))

size.labels = 1
text.size <- 1.3
box.size = 0.5
col.obs <- "white"
col.proj <- "grey75"
col2 <- "grey30"
col = rep(col.obs, times=n.years)
col2 = rep(col2, times=n.years)


X <- pred.seas1
X2 <- pred.seas2

n <- n.years

x.min = 1 ; x.max = n+1
y.min = -5 ; y.max = 7
line.y = y.max*2
y.label = "log(CPUE)"
x.label = "Years"
title = "TITLE"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:n)
{
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i])
}

for (i in 1:n)
{
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = i+0.4, add = T, outpch = NA, boxwex=box.size, col = col2[i])
}

points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:n, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = text.size)
mtext(y.label, line = 3, side = 2, cex = text.size)
# mtext(title, line = 1, side = 3, cex = 1.0) # title(title)

dev.off()
          